const http = require("http");
const open = require("open");
const fetch = require("node-fetch");

// postData fetches by POST and returns the result as JSON
async function postData(url="", data={}) {
    const response = await fetch(url, {
        method: "POST",
        headers: {
            "Content-Type": "application/json",
            "Accept": "application/json"
        },
        body: JSON.stringify(data)
    });
    return await response.json();
}

/**
 * Gets access token 
 * @param {string} clientId 
 * @param {string} redirectUrl 
 */
exports.authenticate = (clientId, redirectUrl) => new Promise( (resolve, reject) => {
    const oauthUrl = "https://anilist.co/api/v2/oauth/token";
    const server = http.createServer(async (req, res) => {
        // when the webpage is opened, there is always a secondary request for favicon.ico
        if (req.url === "/favicon.ico") {
            res.writeHead(200, {"Content-Type": "image/x-icon"});
            res.end();
            
        } else {
            const fullUrl = new URL(req.url, `http://${req.headers.host}`); // must include http://... to create object
            const authCode = fullUrl.searchParams.get("code");
            const apiResponse = await postData(oauthUrl, {
                "grant_type": "authorization_code",
                "client_id": "3328",
                "client_secret": "y9wzVqx88aaPNuqzWP8tradRdqRzqv2B9boQax9I",
                "redirect_uri": "http://127.0.0.1:8080/",
                "code": authCode
            });
            res.writeHead(200, {
                "Content-Type": "text/plain"
            });
            res.write("Done authenticating.  You can close this tab.");
            res.end();
            if (apiResponse.access_token) resolve(apiResponse);
            else reject(Error("Request failed"));
            server.close();
        }
    }).listen(8080);
    open(`https://anilist.co/api/v2/oauth/authorize?client_id=${clientId}&redirect_uri=${redirectUrl}&response_type=code`, {app: "chrome"});
    server.on("close", () => console.log("[anilist-api-auth] server closed"));
});

/* to make an authenticated request, just add the Authorization header
var options = {
  uri: 'https://graphql.anilist.co',
  method: 'POST',
  headers: {
    'Authorization': 'Bearer ' + accessToken,
    'Content-Type': 'application/json',
    'Accept': 'application/json',
  },
  json: {
    'query': query,
  }
};
*/